// attach the microphone ground to a ground socket
// attach the microphone AUD to analog 2 pin
// attach the microphone VCC to digital 2 that will provide 3.3v

#include "TimerOne.h"

#define USE_UART1
//#define FAST_BAUD_RATE
//#define FILTER
#define PERIODIC_CAPTURE   
#define WITH_RECORD_LED

#ifdef WITH_RECORD_LED
#define REC_LED DIGITAL8
#endif

#ifdef FAST_BAUD_RATE
#define BAUD_RATE 125000
#define TIMING_SAMPLING 125 // 8000Hz
#else
#define BAUD_RATE 38400
#define TIMING_SAMPLING 250 // 4000Hz
#endif

#ifdef PERIODIC_CAPTURE
#define CAPTURE_DURATION 15000000UL // in us 15s
#define SLEEP_PERIOD 30000000UL // in us 30s
#define SAMPLE_COUNT_SLEEP SLEEP_PERIOD/TIMING_SAMPLING
#define SAMPLE_COUNT_CAPTURE CAPTURE_DURATION/TIMING_SAMPLING
#endif

#define ULONG_MAX 	                 4294967295UL

uint16_t val;
uint8_t val8bit;

unsigned long lastSleepTime=0;
unsigned long lastWakeupTime=0;
boolean capturingAudio=false;
boolean stopCapture=false;
unsigned long sampleCount=0;

void setup() { 
  // Setting Digital 2 as an output for powering the microphone
  pinMode(DIGITAL2,OUTPUT);
  
#ifdef WITH_RECORD_LED
  pinMode(REC_LED,OUTPUT);
#endif  

#ifdef USE_UART1
  pinMode(DIGITAL6,OUTPUT);
  pinMode(GPRS_PW,OUTPUT);
#else
  // for powering on the XBee module
  pinMode(XBEE_PW, OUTPUT);
#endif

  Timer1.initialize(TIMING_SAMPLING); // 1 interrupt every TIMING_SAMPLING us   
}

void callback () {

  sampleCount++;
  
  if (sampleCount < SAMPLE_COUNT_CAPTURE) {
    
    val = analogRead(ANALOG2) ; // read analog value
    val8bit = ((val >> 2) ) ; // convert into 8 bit

#ifdef FILTER
    // filter silence, very simple method
    if (val8bit > 128) {
#endif      
#ifdef USE_UART1
      // write on UART1, need an XBee module with AP mode 0
      serialWrite(val8bit,1);
#else    
      // write on UART0, need an XBee module with AP mode 0
      serialWrite(val8bit,0);
#endif
#ifdef FILTER
    }
#endif 
  } else {
#ifdef WITH_RECORD_LED
    digitalWrite(REC_LED, LOW);
#endif          
    Timer1.detachInterrupt();
    // we use a variable here because sampling at 8Khz may make the millis() function to not work properly
    stopCapture=true;
  } 

/*  
  if (sampleCount > (SAMPLE_COUNT_CAPTURE+SAMPLE_COUNT_SLEEP)) {
    sampleCount=0;
#ifdef WITH_RECORD_LED
    digitalWrite(REC_LED, HIGH);
#endif     
  }
  */
}

void loop() {
        
        // we have to go to sleep
        if ( (millis() - lastWakeupTime > 15000 || stopCapture) && capturingAudio) {

#ifdef WITH_RECORD_LED
                digitalWrite(REC_LED, LOW);
#endif                
                // shutdown the microphone
                digitalWrite(DIGITAL2,LOW);
                
                // now shutdown the XBee
#ifdef USE_UART1
                digitalWrite(DIGITAL6,LOW);  
                digitalWrite(GPRS_PW,LOW); 
#else
                digitalWrite(XBEE_PW,LOW); 
#endif                
                lastSleepTime = millis();
                capturingAudio = false;
                stopCapture = false;
        }

        // we have to wake up
        if (millis() - lastSleepTime > 5000 && !capturingAudio) {
                
                // power up the microphone
                digitalWrite(DIGITAL2,HIGH);
                
                // power up the XBee
#ifdef USE_UART1
                // set transfert rate
                beginSerial(BAUD_RATE,1);
                Utils.setMuxGPRS();
                digitalWrite(DIGITAL6,HIGH);  
                digitalWrite(GPRS_PW,HIGH); 
#else
                beginSerial(BAUD_RATE,0);
                digitalWrite(XBEE_PW,HIGH); 
#endif  
                // wait a litle bit for the XBee module to start
                delay(500);
                
                sampleCount=0;
                lastWakeupTime = millis();
                capturingAudio = true;
#ifdef WITH_RECORD_LED
                digitalWrite(REC_LED, HIGH);
#endif                
                Timer1.attachInterrupt(callback);
        }        
        
        delay(1000);  
}


